package org.synbiowave.orf;

import java.util.Vector;

import org.synbiowave.workspace.RichSequence;
/**
 * This class finds the ORFs (Open Reading Frames) in the original sequence
 * It has methods to find sequences from all or specific one frame.  
 * @author paradkap
 *
 */
public class ORFFinder {

	// This method returns the sequence of specified length from all the frames(1..6)
	
	public Vector<String> findAll(String sequence, int sequenceLength) {
		try {
			Vector<String> orfs = new Vector<String>();
			Vector<String >orfFrame = new Vector<String>();
			// Calls method FetchFirstFrame that finds first sequence of matching length
			// in that frame for all frames 1..3
			for (int i = 1; i < 4; i++) {
				 orfFrame = find(sequence, i,sequenceLength);
				 for(int j=0; j <orfFrame.size();j++)
				 	orfs.add(orfFrame.get(j));
						
			}
			
			
			// Reverse complement DNA to fetch Frames in reverse strand
			RichSequence seq = new RichSequence();
			seq.setSequence(sequence);
			sequence = seq.getComplement().getSequence();
			// for display only complement
			orfs.add(sequence);
			
	// Calls method FetchFirstFrame that finds first sequence of matching length
			// in that frame for all frames 4..6
			// for computation reverse complement
			seq.Reverse();
			sequence = seq.getSequence();
			for (int i = 1; i < 4; i++) {
				orfFrame = find(sequence, i,sequenceLength);
				for(int j=0; j <orfFrame.size();j++)
				 	orfs.add(orfFrame.get(j));
			}
			
			// Return vector of all the sequences from all the frames.
			return orfs;

		} catch (Exception e) {
			e.printStackTrace();
			System.out.println(e.getMessage());
			return null;
		}
	}

	// This method finds the sequence of specified length in the specified frame 
	public Vector<String> find(String sequence, int frameNo, int sequenceLength) {
		try {
		
		
			// Choose strand based on the frame no specified.
						Vector<Integer> startPos = new Vector<Integer>();
						Vector<String> vecRetFrame=new Vector<String>();
					
						
						
						int frameNum = frameNo > 3 ? frameNo - 3 : frameNo;
						// Fetch start codon in the specific frame
						int len = sequence.length();
						for (int i = (frameNum - 1); i < (len - frameNo - 1); i += 3) {
							String seqStr = sequence.substring(i, i + 3);
							if (seqStr.equalsIgnoreCase("ATG")) {
//								if(frameNo>3)
//									startPos.add(len-i);
//								else
									startPos.add(i);
							}
							if (seqStr.equalsIgnoreCase("TAA")
									|| seqStr.equalsIgnoreCase("TAG")
									|| seqStr.equalsIgnoreCase("TGA")) {
								
								int endPos;
								endPos = i + 3;
								if(startPos!=null && startPos.size()>0){
									int frameStart = startPos.get(0);
									if(endPos - frameStart >= sequenceLength){
												vecRetFrame.add(frameNo+"#"+ frameStart + "#" + sequence.substring(frameStart,endPos) + "#"+ endPos);
									}
									startPos.clear();
							}
							}
							
						}

						
						
						
			// Return the vector of sequences returned by the method.
			return vecRetFrame;

		} catch (Exception e) {
			return null;
		}
	}

	
}
