package org.synbiowave.translate;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import org.biojava.bio.seq.DNATools;
import org.biojava.bio.symbol.SymbolList;
import org.synbiowave.AbstractSynBioWaveRobot;
import org.synbiowave.biojava.SequenceUtils;
import org.synbiowave.orf.ORFFinder;
import org.synbiowave.utils.SequenceColorUtils;
import org.synbiowave.workspace.RichSequence;
import org.synbiowave.workspace.Sequence;

import com.google.wave.api.Blip;
import com.google.wave.api.event.Event;

public class Translator {
	static Hashtable<String,String> codons;
	
	public static void initiateCodonTable(){
		codons = new Hashtable<String,String>();
		codons.put("GCT", "A");
		codons.put("GCC", "A");
		codons.put("GCA", "A");
		codons.put("GCG", "A");
		codons.put("GAT", "D");
		codons.put("GAC", "D");
		codons.put("TTT", "F");
		codons.put("TTC", "F");
		codons.put("CAT", "H");
		codons.put("CAC", "H");
		codons.put("AAA", "K");
		codons.put("AAG", "K");
		codons.put("ATG", "M");
		codons.put("CCT", "P");
		codons.put("CCC", "P");
		codons.put("CCA", "P");
		codons.put("CCG", "P");
		codons.put("CGT", "R");
		codons.put("CGC", "R");
		codons.put("CGA", "R");
		codons.put("CGG", "R");
		codons.put("AGA", "R");
		codons.put("AGG", "R");
		codons.put("ACT", "T");
		codons.put("ACC", "T");
		codons.put("ACA", "T");
		codons.put("ACG", "T");
		codons.put("TGG", "W");
		codons.put("TGT", "C");
		codons.put("TGC", "C");
		codons.put("GAA", "E");
		codons.put("GAG", "E");
		codons.put("GGT", "G");
		codons.put("GGC", "G");
		codons.put("GGA", "G");
		codons.put("GGG", "G");
		codons.put("ATT", "I");
		codons.put("ATC", "I");
		codons.put("ATA", "I");
		codons.put("TTA", "L");
		codons.put("TTG", "L");
		codons.put("CTT", "L");
		codons.put("CTC", "L");
		codons.put("CTA", "L");
		codons.put("CTG", "L");
		codons.put("AAT", "N");
		codons.put("AAC", "N");
		codons.put("CAA", "Q");
		codons.put("CAG", "Q");
		codons.put("TCT", "S");
		codons.put("TCC", "S");
		codons.put("TCA", "S");
		codons.put("TCG", "S");
		codons.put("AGT", "S");
		codons.put("AGC", "S");
		codons.put("GTT", "V");
		codons.put("GTC", "V");
		codons.put("GTA", "V");
		codons.put("GTG", "V");
		codons.put("TAT", "Y");
		codons.put("TAC", "Y");
		codons.put("TAA", "*");
		codons.put("TGA", "*");
		codons.put("TAG", "*");
	}
	
	public static void init(List<Sequence> sequences,AbstractSynBioWaveRobot servlet, Event event, int frameNo,	int sequenceLength) {
		try{
			ORFFinder objORF = new ORFFinder();
			Sequence sequence = sequences.get(0);
			String strSequence = sequence.getSequence();
			List<Sequence> lstSequences = new ArrayList<Sequence>();
			lstSequences.add(sequence);
			Blip frmBlip = event.getWavelet().reply("\n");
		//	Blip frmBlip=null;
			Vector<String> seqNames = new Vector<String>();
			HashMap<String,String> mapFrames = new HashMap<String,String>();
			Vector<String> vecFrames = new Vector<String>();
			char[]a = new char[sequence.getSequence().length()];
			Arrays.fill(a, '-');
			String str = new String(a);
			if(frameNo==0){//Search all ORFs
				
			
				for(int j=1; j < 4; j++){
				Vector<String> vecStr = objORF.find(strSequence,j, sequenceLength);
				if(vecStr.size()!=0)
				{
					mapFrames.put("Frame_"+j, str);
					for(int k=0; k < vecStr.size();k++)
						vecFrames.add(vecStr.get(k));
					
				}
				else
					vecFrames.add(j+"#NO ORF of speicified length");
				}

				SymbolList syml = DNATools.createDNA(strSequence);
				String strCompl = DNATools.complement(syml).seqString().toUpperCase();
				vecFrames.add("COMPLEMENT#"+strCompl);
				mapFrames.put("COMPLEMENT",strCompl);
				
				// reverse complement for computation
				strSequence = DNATools.reverseComplement(syml).seqString().toUpperCase();
				for(int j=4; j < 7; j++){
					
					Vector<String> vecStr = objORF.find(strSequence,j, sequenceLength);
					if(vecStr.size()!=0)
					{
						mapFrames.put("Frame_"+j, str);
						for(int k=0; k < vecStr.size();k++)
							vecFrames.add(vecStr.get(k));
						
					}
					else{
						mapFrames.put("Frame_"+j, "NO ORF of speicified length");
						vecFrames.add(j+"#NO ORF of speicified length");
					}
					}
				
			}
			else{//Search specific ORF
				mapFrames.put("Frame_"+frameNo, str);
				if(frameNo>3){
					
					SymbolList syml = DNATools.createDNA(strSequence);
					String strCompl = DNATools.complement(syml).seqString();
					mapFrames.put("COMPLEMENT", strCompl);
					vecFrames.add("COMPLEMENT#"+ strCompl);
					// reverse complement for computation
					strSequence = DNATools.reverseComplement(syml).seqString();
				}
								
				Vector<String> vecStr = objORF.find(strSequence,frameNo, sequenceLength);
				if(vecStr.size()!=0)
				{
					for(int k=0; k < vecStr.size();k++)
						vecFrames.add(vecStr.get(k));
					
				}
				else{
					vecFrames.add(frameNo+"#NO ORF of speicified length");
					mapFrames.put("Frame_"+frameNo, "NO ORF of speicified length");
				}
				
			}
			String translated = "";
			Vector<String> vecFrameNames = new Vector<String>();
			
			// Initiate the translation hashtable
			initiateCodonTable();
			for(int i=0; i < vecFrames.size(); i++){
				String strSeq = vecFrames.get(i);
				int frameNum=-1;
				String seqName="";
				if(strSeq.indexOf("#")!=-1){
					
					String[] strArr = strSeq.split("#");
					if(strArr!=null && strArr.length==2){
						if(strArr[0].equals("COMPLEMENT")){
						seqName = strArr[0];
						strSeq = strArr[1];
						}
						else{
						frameNum = Integer.parseInt(strArr[0]);
						seqName = "Frame_" + frameNum;
						strSeq = strArr[1];
						}
					}
					if(strArr!=null && strArr.length==4){
							int start = Integer.parseInt(strArr[1]);
							int end = Integer.parseInt(strArr[3]);
							frameNum = Integer.parseInt(strArr[0]);
						
							seqName = "Frame_" + frameNum;
						
							translated = translate(strArr[2],frameNum);
							char[] tArr = translated.toCharArray();
							char[] retSeq;
							if(frameNum > 3)
								retSeq = new StringBuffer(mapFrames.get(seqName)).reverse().toString().toCharArray();
							else
								retSeq = mapFrames.get(seqName).toCharArray();
							
							int endloop = tArr.length;

							for(int p=start,q=0; q<endloop;q++, p+=3){
								retSeq[p] = '-';
								retSeq[p+1] = tArr[q];
								retSeq[p+2] = '-';
								
							}

							if(frameNum > 3)
								translated = new StringBuffer(new String(retSeq)).reverse().toString();
							else
								translated = new String(retSeq);
						
					}
					else
					{
						translated = strSeq;
					}
					if(!vecFrameNames.contains(seqName))
						vecFrameNames.add(seqName);
					
					mapFrames.put(seqName, translated);
					
				}
//				else{
//					mapFrames.put(frameNum,vecFrames.get(i));
//				}
					
			}
			
			
			// Add from map to list
			for(int j=0;j<vecFrameNames.size();j++){
				String frame = vecFrameNames.get(j);
				String strframe = mapFrames.get(frame);
				Sequence seq = new Sequence(frame,strframe);
				lstSequences.add(seq);
			}
			SequenceColorUtils.showGadgetView(frmBlip, lstSequences);
		}
		catch(Exception e){
			event.getWavelet().reply("\n: Error translating ORF " + e.getMessage());
		}
	}
	
	public static String translate(String frame,int frameNo){
		if(codons==null || codons.size()==0)
			initiateCodonTable();
		StringBuffer retTrans = new StringBuffer("");
		for(int i=0; i<=frame.length()-3;i+=3){
			String substr = frame.substring(i, i+3);
	//		String str = frameNo>3?new StringBuffer(substr).reverse().toString():substr;
		
			if(codons.get(substr.toUpperCase())!=null)
			retTrans.append(codons.get(substr.toUpperCase()));
			
		}
		return retTrans.toString();
	}
}
