package org.synbiowave.utils;

import com.google.wave.api.Annotation;
import com.google.wave.api.Annotations;
import com.google.wave.api.Blip;
import com.google.wave.api.BlipContentRefs;
import com.google.wave.api.Gadget;
import com.google.wave.api.Range;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.biojava.bio.seq.DNATools;
import org.biojava.bio.seq.ProteinTools;
import org.biojava.bio.seq.RNATools;
import org.biojava.bio.symbol.IllegalSymbolException;
import org.synbiowave.workspace.Sequence;

public class SequenceColorUtils
{
  private static char[][] aColorCode = { { 'Y', 'F', 'A', 'I', 'L', 'M', 'V' }, 
    { 'W' }, 
    { 'G' }, 
    { 'P' }, 
    { 'S', 'D', 'E', 'T' }, 
    { 'R', 'N', 'K' }, 
    { 'H' }, 
    { 'Q' }, 
    { 'C' } };

  private static String[] aColors = { "green", "mediumturquoise", "magenta", "gold", "red", "blue", "cornflowerblue", "gray", "saddlebrown" };

  private static char[][] nColorCode = { { 'U', 'T' }, 
    { 'A' }, 
    { 'G' }, 
    { 'C' } };

  private static String[] nColors = { "red", "green", "black", "blue" };
  private static final char seperator = '#';

  public static String determineType(String sequence)
  {
    String type = "DNA";
    try { DNATools.createDNA(sequence);
    } catch (IllegalSymbolException e) {
      type = "RNA";
      try { RNATools.createRNA(sequence);
      } catch (IllegalSymbolException se) {
        type = "PROTEIN-TERM";
        try { ProteinTools.createProtein(sequence);
        } catch (IllegalSymbolException ise) {
          type = "null";
        }
      }
    }return type;
  }

  public static String getColor(char letter, String type)
  {
    if (type == null) return "black";
    char[][] colorCode = nColorCode;
    String[] colors = nColors;
    String id="n";
    if (type.contains("PROTEIN")) {
      colorCode = aColorCode;
      colors = aColors;
      id = "a";
    }for (int r = 0; r < colorCode.length; ++r)
      for (int c = 0; c < colorCode[r].length; ++c)
        if (colorCode[r][c] == Character.toUpperCase(letter)) return id+(r+1);
    return "black";
  }

  public static ArrayList<String> getScale(int length)
  {
    ArrayList<String> scalings = new ArrayList();
    StringBuffer scale = new StringBuffer();
    StringBuffer numbers = new StringBuffer();
    int wait = 0;
    for (int i = 1; i < length + 1; ++i) {
      if (i % 5 == 0) {
        scale.append('|');
        numbers.append(i);
        wait = String.valueOf(i).length() - 1;
      } else {
        scale.append('\'');
        if (wait == 0) numbers.append(' '); else
          --wait; 
      }
      if (i % 60 == 0) {
        scalings.add("\n" + numbers.append("\n"));
        scale = new StringBuffer();
        numbers = new StringBuffer();
        wait = 0;
      }
    }scalings.add("\n" + numbers.append("\n"));
    return scalings;
  }

  public static String getHTMLView(String sequence)
  {
    String type = determineType(sequence);
    StringBuffer scale = new StringBuffer();
    StringBuffer numbers = new StringBuffer();
    int wait = 0;
    for (int i = 1; i < sequence.length() + 1; ++i)
      if (i % 5 == 0) {
        scale.append('|');
        numbers.append(i);
        wait = String.valueOf(i).length() - 1;
      } else {
        scale.append('\'');
        if (wait == 0) numbers.append("&nbsp;"); else
          --wait; 
      }
    String htmlView = "(br /)" + scale + "(br /)" + numbers + "(br /)";
    String id = "n";
    char[][] colorCode = nColorCode;
    String[] colors = nColors;
    if (type.contains("PROTEIN")) {
      colorCode = aColorCode;
      colors = aColors;
      id = "a";
    }char[] sa = sequence.toUpperCase().toCharArray();
 //   htmlView = sequence.toUpperCase();
    for (char sc : sa)
      for (int r = 0; r < colorCode.length; ++r)
        for (int c = 0; c < colorCode[r].length; ++c)
          if (colorCode[r][c] == sc)
            htmlView = htmlView + "(SP#" + id +(r+1) + ";')" + sc + "(/span)";
    return htmlView + "(br /)(br /)";
  }

  public static String getHTMLView(List<Sequence> sequences)
  {
    int maxLength = 0;
    for (Sequence sequence : sequences)
    {
      if (sequence.getSequence().length() <= maxLength)
        continue;
      maxLength = sequence.getSequence().length();
    }

    StringBuffer scale = new StringBuffer();
    StringBuffer numbers = new StringBuffer();

    int wait = 0;
    for (int i = 1; i < maxLength + 1; ++i)
      if (i % 5 == 0) {
        scale.append('|');
        numbers.append(i);
        wait = String.valueOf(i).length() - 1;
      } else {
        scale.append('\'');
        if (wait == 0) numbers.append("&nbsp;"); else
          --wait; 
      }
    StringBuilder htmlView = new StringBuilder("(br /)" + scale + "(br /)" + numbers + "(br /)");
    for (Sequence sequence : sequences)
    {
      for (char sc : sequence.getSequence().toUpperCase().toCharArray())
      {
    	
        htmlView.append("(SP#" + getColor(sc, sequence.getType()) + ";\")" + sc + "(#SP)");
      }
      htmlView.append("(br /)");
    }
    return htmlView.toString() + "(br /)";
  }

  public static void addScaling(Blip blip, Range range)
  {
    int index = range.getStart();
    for (String scale : getScale(range.getEnd() - range.getStart())) {
      blip.at(index).insert(new String[] { "\n\n" });
      index += 2;
      blip.at(index).insert(new String[] { scale });
      blip.range(index, index + scale.length()).annotate("scale", new String[] { "default" });
      index += scale.length();
      index += scale.indexOf("\n");
    }if (index < blip.length()) blip.at(index).insert(new String[] { "\n\n" });
    blip.range(range.getStart(), index).annotate("style/fontFamily", new String[] { "monospace" });
    blip.range(range.getStart(), index).annotate("sequence", new String[] { "default" });
  }

  public static void addSequenceColors(Blip blip, String type)
  {
    for (Annotation sequence : blip.getAnnotations().get("sequence"))
      for (int index = sequence.getRange().getStart(); index < sequence.getRange().getEnd(); ++index)
        if (Character.isLetter(blip.getContent().charAt(index)))
          blip.at(index).annotate("style/color", new String[] { getColor(blip.getContent().charAt(index), type) });
  }

  /** @deprecated */
  public static void showGadgetView(Blip blip, Map<String, String> sequences)
  {
    List dummyList = new ArrayList();
    for (String key : sequences.keySet())
    {
      dummyList.add(new Sequence(key, (String)sequences.get(key)));
    }

    String htmlViewSequences = getHTMLView(dummyList);
    String names = "(br /)(br /)(br /)";
    String name;
    for (Iterator localIterator2 = sequences.keySet().iterator(); localIterator2.hasNext(); names = names + name + "(br /)") name = (String)localIterator2.next();
    Gadget seqView = new Gadget("http://samnode.de/SequenceView.xml");
    seqView.setProperty("names", names + "(br /)(br /)");
    seqView.setProperty("sequences", htmlViewSequences);
    blip.append(seqView);
  }

  public static void showGadgetView(Blip blip, List<Sequence> sequences)
  {
    String htmlViewSequences = getHTMLView(sequences);
    StringBuilder names = new StringBuilder("(br /)(br /)(br /)(br /)(br /)");
    for (Sequence seq : sequences)
    {
      names.append(seq.getName() + "(br /)");
    }
    Gadget seqView = new Gadget("http://translate-beta.appspot.com/SequenceView.xml");
    seqView.setProperty("names", names + "(br /)(br /)");
    seqView.setProperty("sequences", htmlViewSequences);
    blip.append(seqView);
  }

  /** @deprecated */
  public static void showGadgetView(Blip blip, String name, String sequence)
  {
    Map sequences = new HashMap();
    sequences.put(name, sequence);
    showGadgetView(blip, sequences);
  }

  public static String Serialize(Sequence seq)
  {
    String output = "";
    output = output + seq.getName();
    output = output + '#';
    output = output + seq.getAuthor();
    output = output + '#';
    if (seq.getType() != null)
    {
      output = output + seq.getType();
    }
    else {
      output = output + "OTHER";
    }
    output = output + '#';
    output = output + seq.getSequence();
    output = output + '#';

    return output;
  }

  public static Sequence Deserialize(String input)
  {
    String[] parts = input.split(Character.toString('#'));
    Sequence output = new Sequence();
    try {
      output.setName(parts[0]);
      output.setAuthor(parts[1]);
      output.setType(parts[2]);
      output.setSequence(parts[3]);
    }
    catch (Exception localException)
    {
    }
    return output;
  }
}